import React, {
  useState,
  useCallback,
  useEffect,
} from 'react';
import ReactDOM from 'react-dom';
import ReactDropdown from 'react-dropdown';
import {
  csv,
  scaleLinear,
  max,
  format,
  extent,
  scaleOrdinal,
} from 'd3';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';
import { Dropdown } from './Dropdown';
import { ColorLegend } from './ColorLegend';
const csv_url =
  'https://gist.githubusercontent.com/curran/0ac4077c7fc6390f5dd33bf5c06cb5ff/raw/605c54080c7a93a417a3cea93fd52e7550e76500/UN_Population_2019.csv';

const width = 800;
const menuHeight = 80;
const height = 500 - menuHeight;
const margin = {
  top: 20,
  right: 30,
  bottom: 65,
  left: 90,
};
const xAxisLabelOffset = 50;
const yAxisLabelOffset = 45;

const fadeOpacity = 0.2;

const attributes = [
  {
    value: 'sepal_length',
    label: 'Sepal Length',
  },
  { value: 'sepal_width', label: 'Sepal Width' },
  {
    value: 'petal_length',
    label: 'Petal Length',
  },
  { value: 'petal_width', label: 'Petal Width' },
  { value: 'species', label: 'Species' },
];

const getLabel = (value) => {
  for (let i = 0; i < attributes.length; i++) {
    if (attributes[i].value === value) {
      return attributes[i].label;
    }
  }
};

const App = () => {
  const data = useData();
  const initialXAttribute = 'sepal_length';
  const [xAttribute, setXAttribute] = useState(
    initialXAttribute
  );
  
  const [hoveredValue, setHoveredValue] = useState(null);
  
  const xValue = (d) => d[xAttribute];
  const xAxisLabel = getLabel(xAttribute);

  const initialYAttribute = 'sepal_width';
  const [yAttribute, setYAttribute] = useState(
    initialYAttribute
  );
  const yValue = (d) => d[yAttribute];
  const yAxisLabel = getLabel(yAttribute);

  if (!data) {
    return <pre>Loading...</pre>;
  }
  
  const innerHeight =
    height - margin.top - margin.bottom;
  const innerWidth =
    width - margin.left - margin.right - 100;

  const circleRadius = 7;
  
  const siFormat = format('.2s');
  const xAxisTickFormat = (tickValue) =>
    siFormat(tickValue).replace('G', 'B');

  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([0, innerHeight])
    .nice();

  const colorValue = (d) => d.species;
  
  const filteredData = data.filter(d => hoveredValue === colorValue(d)); 

  const colorLegendLabel = 'Species'
  
  const colorScale = scaleOrdinal()
    .domain(data.map(colorValue))
    .range(['#F2DA57', '#F6B656', '#42A5B3']);

  return (
    <>
      <div className="menus-container">
        <span className="dropdown-label">
          {' '}
          X{' '}
        </span>
        <ReactDropdown
          options={attributes}
          value={xAttribute}
          onChange={({ value }) =>
            setXAttribute(value)
          }
        />
        <span className="dropdown-label">
          {' '}
          Y{' '}
        </span>
        <ReactDropdown
          options={attributes}
          value={yAttribute}
          onChange={({ value }) =>
            setYAttribute(value)
          }
        />
      </div>
      <svg width={width} height={height}>
        <g
          transform={`translate(${margin.left},${margin.top})`}
        >
          <AxisBottom
            tickOffset={5}
            tickFormat={xAxisTickFormat}
            xScale={xScale}
            innerHeight={innerHeight}
          />
          <AxisLeft
            tickOffset={5}
            yScale={yScale}
            innerWidth={innerWidth}
          />
          <text
            className="axis-label"
            x={innerWidth / 2}
            textAnchor="middle"
            y={innerHeight + xAxisLabelOffset}
          >
            {xAxisLabel}
          </text>
          <text
            className="axis-label"
            textAnchor="middle"
            transform={`translate(${-yAxisLabelOffset},${
              innerHeight / 2
            }) rotate(-90)`}
          >
            {yAxisLabel}
          </text>
          <g
            transform={`translate(${
              innerWidth + 30},
              ${innerHeight - 300})`}
          >
            <text
            x={35}
            y={-25}
            className="axis-label"
            textAnchor="middle"
          	>
            {colorLegendLabel}
          </text>
              <ColorLegend
                colorScale={colorScale}
                tickSpacing={25}
                tickSize={circleRadius}
                tickTextOffSet={20}
                onHover = {setHoveredValue}
                hoveredValue = {hoveredValue}
                fadeOpacity = {fadeOpacity}
              />
          </g>
          <g opacity={hoveredValue ? fadeOpacity : 1}>
            <Marks 
            data={data}
            xScale={xScale}
            yScale={yScale}
            xValue={xValue}
            yValue={yValue}
            toolTipFormat={xAxisTickFormat}
            circleRadius={circleRadius}
            colorScale={colorScale}
            colorValue={colorValue}
          	/>
          </g>
          <Marks
            data={filteredData}
            xScale={xScale}
            yScale={yScale}
            xValue={xValue}
            yValue={yValue}
            toolTipFormat={xAxisTickFormat}
            circleRadius={circleRadius}
            colorScale={colorScale}
            colorValue={colorValue}
          />
        </g>
      </svg>
    </>
  );
};

const rootElement = document.getElementById(
  'root'
);
ReactDOM.render(<App />, rootElement);
